/*
  Acquire and save data from PDS5022S digital strage oscilloscope

  2010-03-26
  T. Nakagawa
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <usb.h>


#define FILE_DEFAULT "owonsave"
#define FILE_SIZE 256

#define VENDOR_ID 0x5345
#define PRODUCT_ID 0x1234
#define CMD_SIZE 256
#define ENDPOINT_WRITE 0x03
#define ENDPOINT_READ 0x81
#define CMD_START "START"
#define CMD_TIMEOUT 3000
#define READ_TIMEOUT 5000


int main(int argc, char **argv) {
  struct usb_bus *bus;
  struct usb_device *dev;
  usb_dev_handle *dh;
  char file[FILE_SIZE];
  int rv;
  unsigned char cmd[CMD_SIZE];
  int buf_size;
  unsigned char *buf;
  FILE *fp;

  if (argc == 1) {
    strcpy(file, FILE_DEFAULT);
  } else if (argc == 2) {
    if (strlen(argv[1]) + 4 + 1 > FILE_SIZE) {
      fprintf(stderr, "ERROR: too long file name");
      return 1;
    }
    strcpy(file, argv[1]);
  } else {
    fprintf(stderr, "usage: %s [<file name>]\n", argv[0]);
    return 1;
  }

  usb_init();

  usb_find_busses();
  usb_find_devices();
  for (bus = usb_get_busses(); bus != NULL; bus = bus->next) {
    for (dev = bus->devices; dev != NULL; dev = dev->next) {
      if (dev->descriptor.idVendor == VENDOR_ID && dev->descriptor.idProduct == PRODUCT_ID) {
        break;
      }
    }
    if (dev != NULL) break;
  }
  if (bus == NULL) {
    fprintf(stderr, "ERROR: cannot find a device");
    return 1;
  }

  if ((dh = usb_open(dev) )== NULL) {
    fprintf(stderr, "ERROR: cannot open a device");
    return 1;
  }

  usb_set_configuration(dh, dev->config->bConfigurationValue);
  usb_claim_interface(dh, dev->config->interface->altsetting->bInterfaceNumber);
  if ((rv = usb_set_altinterface(dh, dev->config->interface->altsetting->bInterfaceNumber)) < 0) goto quit;
  usb_clear_halt(dh, ENDPOINT_WRITE);
  usb_clear_halt(dh, ENDPOINT_READ);

  if ((rv = usb_bulk_write(dh, ENDPOINT_WRITE, CMD_START, strlen(CMD_START), CMD_TIMEOUT)) < 0) goto quit;

  if ((rv = usb_bulk_read(dh, ENDPOINT_READ, (char *)cmd, CMD_SIZE, CMD_TIMEOUT)) < 0) goto quit;

  buf_size = cmd[0] + cmd[1] * 0x100 + cmd[2] * 0x10000 + cmd[3] * 0x1000000;
  buf = malloc(sizeof(char) * buf_size);
  if (buf == NULL) {
    fprintf(stderr, "ERROR: cannot allocate memory\n");
    goto quit;
  }

  if ((rv = usb_bulk_read(dh, ENDPOINT_READ, (char *)buf, buf_size, READ_TIMEOUT)) < 0) goto quit;

  if (strncmp((char *)buf, "BM", 2) == 0) {
    strcat(file, ".bmp");
  } else if (strncmp((char *)buf, "SPB", 3) == 0) {
    strcat(file, ".bin");
  } else {
    fprintf(stderr, "ERROR: unknown data type");
    goto quit;
  }
  if ((fp = fopen(file, "wb")) == NULL) {
    fprintf(stderr, "ERROR: cannot open a file");
    goto quit;
  }
  fwrite(buf, sizeof(char), rv, fp);
  fclose(fp);
  free(buf);

  usb_release_interface(dh, dev->config->interface->altsetting->bInterfaceNumber);

quit:
  if (rv < 0) {
    fprintf(stderr, "ERROR: USB: %s\n", strerror(-rv));
  }

  usb_reset(dh);
  usb_close(dh);

  return 0;
}
